package org.nakedobjects.nos.store.hibernate;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.collection.PersistentCollection;
import org.nakedobjects.nof.reflect.java.reflect.ClassStrategy;
import org.nakedobjects.nof.reflect.java.reflect.DefaultClassStrategy;
import org.nakedobjects.nof.reflect.java.reflect.DelegatingClassStrategy;


/**
 * A decorator for an existing {@link ClassStrategy} to add features required by the Hibernate Object store.
 */
public class HibernateClassStrategy extends DelegatingClassStrategy {

    public HibernateClassStrategy() {
        this(new DefaultClassStrategy());
    }

    public HibernateClassStrategy(final ClassStrategy underlying) {
        super(underlying);
    }

    public Class getClass(final Class cls) {
        /*
         * TODO this mechanism will not work properly if the collection classes are subclassed by the
         * programmer (which will be done when actions need to be added to classes. The purpose of this
         * hierarchy will then be gone.
         * 
         * Therefore this should all be replaced.
         */
        if (PersistentCollection.class.isAssignableFrom(cls)) {
            if (List.class.isAssignableFrom(cls)) {
                return List.class;
            }
            if (Set.class.isAssignableFrom(cls)) {
                return Set.class;
            }
            if (Map.class.isAssignableFrom(cls)) {
                return Map.class;
            }
            return cls;
        }
        // if (HibernateProxy.class.isAssignableFrom(cls)) {
        // return cls.getSuperclass();
        // }
        return super.getClass(cls);
    }

}
// Copyright (c) Naked Objects Group Ltd.
