package org.nakedobjects.nos.store.hibernate;

import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.java.JavaDomainObjectContainer;

import org.apache.log4j.Logger;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.engine.SessionImplementor;
import org.hibernate.proxy.HibernateProxy;
import org.hibernate.proxy.LazyInitializer;


// TODO optimise for Hibernate
public class HibernateContainer extends JavaDomainObjectContainer {
    private static final Logger LOG = Logger.getLogger(HibernateContainer.class);

    public void resolve(final Object parent, Object field) {
        // Don't do any resolving etc if we are in the commit phase of a transaction
        // if (HibernateUtil.isFlushing()) {
        // return;
        // }

        if (field == null) {
            // TODO in Hibernate object references and collections will be proxied or wrapped,
            // hence null actually means they have been resolved - so this may be called
            // for null object reference!!
            super.resolve(parent);
            // objectPersistor().resolveImmediately(adapter);
            return;
        }
        // REVIEW Why is work being done here? The not-null field indicates that the parent object is already loaded.
        
        // REVIEW the adapter should already exists
        // NakedObject adapter =
        // NakedObjectsContext.getObjectLoader().getAdapterForElseCreateAdapterForTransient(parent);
        NakedObject adapter = NakedObjectsContext.getObjectLoader().getAdapterFor(parent);
        if (adapter == null) {
            throw new NakedObjectRuntimeException("No adapter for " + parent);
        }

        ResolveState resolveState = adapter.getResolveState();
        // TODO check this line?
        if (!resolveState.isResolvable(ResolveState.RESOLVING)) {
            return;
        }
        // TODO would really like to push this into objectPersistor, but current setup
        // requires a NakedObject field, which we can't get!

        // TODO loadEvent will create adapter for proxy, but collection is a different matter.
        // if adapter exists for collection it will be set to resolved, but but can't create new adapter
        // as we can't get the field
        if (!Hibernate.isInitialized(field)) {
            // This method may not be running within the scope of a transaction, so
            // make sure one is active
            NakedObjectsContext.getObjectPersistor().startTransaction();
            try {
                Session session = HibernateUtil.getCurrentSession();
                if (field instanceof org.hibernate.collection.PersistentCollection) {
                    session.lock(parent, org.hibernate.LockMode.NONE);
                    Hibernate.initialize(field);
                } else {
                    LazyInitializer lazy = ((HibernateProxy) field).getHibernateLazyInitializer();
                    lazy.setSession((SessionImplementor) session);
                    lazy.initialize();
                }
            } catch (RuntimeException e) {
                NakedObjectsContext.getObjectPersistor().abortTransaction();
                throw e;
            }
            NakedObjectsContext.getObjectPersistor().endTransaction();
            if (LOG.isDebugEnabled()) {
                LOG.debug("Container resolved field of type " + field.getClass() + " for parent " + parent + ", state="
                        + resolveState);
            }
        }
    }
}
// Copyright (c) Naked Objects Group Ltd.
