package org.nakedobjects.nos.store.hibernate;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.noa.reflect.NakedObjectReflector;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.persist.ObjectPersistorLogger;
import org.nakedobjects.nof.core.system.ObjectLoaderInstaller;
import org.nakedobjects.nof.core.system.ObjectPersistorInstaller;
import org.nakedobjects.nof.core.util.NakedObjectConfiguration;
import org.nakedobjects.nof.persist.PersistAlgorithm;
import org.nakedobjects.nof.persist.objectstore.NakedObjectStore;
import org.nakedobjects.nof.persist.objectstore.ObjectStorePersistor;
import org.nakedobjects.nof.reflect.java.reflect.JavaReflector;
/**
 * Installs the Hibernate object store.
 */
public class HibernatePersistorInstaller implements ObjectPersistorInstaller {
	private static final Logger LOG = Logger.getLogger(HibernatePersistorInstaller.class);

	public NakedObjectPersistor createObjectPersistor(final NakedObjectConfiguration configuration) {
		LOG.info("installing " + this.getClass().getName());

		NakedObjectStore objectStore;
		if (configuration.getBoolean(HibernateUtil.PROPERTY_PREFIX + "saveImmediate",true)) {
			objectStore = new HibernateObjectStoreImmediate();
		} else {
			objectStore = new HibernateObjectStore();
		}
		// REVIEW this will replace the saveImmediate setting above
		if (configuration.getBoolean(HibernateUtil.PROPERTY_PREFIX + "remapping",false)) {
			objectStore = new RemappingHibernateObjectStore(objectStore);
		}

        String algorithm = configuration.getString(HibernateUtil.PROPERTY_PREFIX + "persistAlgorithm");
        PersistAlgorithm persistAlgorithm;
        if ("simple".equals(algorithm)) {
			persistAlgorithm = new SimplePersistAlgorithm();
        } else {
        	persistAlgorithm = new TwoPassPersistAlgorithm();
        }

		ObjectStorePersistor persistor = new ObjectStorePersistor();
		persistor.setObjectStore(objectStore);
		persistor.setPersistAlgorithm(persistAlgorithm);
		// persistor.setCheckObjectsForDirtyFlag(true); // TODO can Hibernate do this?
		persistor.setOidGenerator(new HibernateOidGenerator());
        
        // TODO pull this information from the persistor
        NakedObjectsContext.getConfiguration().add("oid-generator", HibernateOidGenerator.class.getName());

		return new ObjectPersistorLogger(persistor);
	}
    
    public String getName() {
        return "hibernate";
    }

    public ObjectLoaderInstaller getPreferredObjectLoaderInstaller() {
        return new HibernateObjectLoaderInstaller();
    }

    public void setUpReflector(NakedObjectReflector reflector) {
        if (reflector instanceof JavaReflector) {
            ((JavaReflector) reflector).setClassStrategy(new HibernateClassStrategy());
        }
    }
}
// Copyright (c) Naked Objects Group Ltd.
