package org.nakedobjects.nos.store.hibernate.property;

import java.util.HashMap;

import org.nakedobjects.noa.reflect.NakedObjectField;

/**
 * Factory to return a PropertyConverter for conversion between a specific 
 * NakedObjects value holder type and standard Java data type.
 * Additional Converters can be added by calling getInstance().add(..)
 */

public class ConverterFactory {
	private HashMap converters = new HashMap();
//	public static final PropertyConverter defaultConverter = new DefaultConverter();
	private static ConverterFactory instance = new ConverterFactory();

	private ConverterFactory() {
		// TODO: requires org.nakedobjects.applib-original, which isn't
		// in the current build.  Add back in when available?  Or just
		// leave and make apps add the converters back in by hand?
		/*
		add(Color.class, new IntegerToColorConverter());
		add(Date.class, new DateToDateConverter());
		add(Logical.class, new BooleanToLogicalConverter());
		add(Money.class, new BigDecimalToMoneyConverter());
		addDefault(MultilineTextString.class);
		addDefault(Option.class);
		addDefault(Password.class);
		add(Percentage.class, new BigDecimalToPercentageConverter());
		add(SerialNumber.class, new LongToSerialNumberConverter());
		addDefault(SimpleState.class);
		addDefault(TextString.class);
		add(Time.class, new TimeToTimeConverter());
		add(TimeStamp.class, new TimestampToTimeStampConverter());
		addDefault(URLString.class);
		add(WholeNumber.class, new IntegerToWholeNumberConverter());
		add(XmlString.class, new TextToXmlStringConverter());
		*/
	}

	/**
	 * Return a PropertyConverter which converts from a value holder
	 *  to a persistent class.
	 * @param nakedClass Class of the value holder to convert from
	 * 			eg WholeNumber.class
	 */
	public PropertyConverter getConverter(final Class nakedClass) {
	    return getConverter(nakedClass.getName());
	}

	/**
	 * Return a PropertyConverter which converts from a value holder
	 *  to a persistent class.
	 * @param nakedClass name of the value holder to convert from
	 * @return the PropertyConverter, or null if none is registered
	 */
	public PropertyConverter getConverter(final String nakedClass) {
	    return (PropertyConverter) converters.get(nakedClass);
	}
	public PropertyConverter getConverter(final NakedObjectField field) {
		return getConverter(field.getSpecification().getFullName());
	}

	/**
	 * Add a converter for the specified value type
	 */
	public void add(final Class valueType, final PropertyConverter converter) {
	    if(!converters.containsKey(valueType.getName())) {
	    	converters.put(valueType.getName(), converter);    
	    }
	}

	/**
	 * Return the singleton ConverterFactory
	 */
	public static ConverterFactory getInstance() {
		return instance;
	}
}
// Copyright (c) Naked Objects Group Ltd.
