package org.nakedobjects.nos.store.hibernate.service;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;

import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.persist.InstancesCriteria;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.remote.data.CriteriaData;
import org.nakedobjects.nof.reflect.remote.data.Data;
import org.nakedobjects.nof.reflect.remote.data.KnownObjects;
import org.nakedobjects.nof.reflect.remote.data.ObjectEncoder;
import org.nakedobjects.nos.remote.command.marshal.AbstractCriteriaEncoder;
import org.nakedobjects.nos.store.hibernate.QueryPlaceholder;


public class HibernateQueryEncoder extends AbstractCriteriaEncoder {
    public CriteriaData createData(final InstancesCriteria criteria, final ObjectEncoder encoder) {
        final HibernateQueryCriteria hibernateCriteria = (HibernateQueryCriteria) criteria;
        final List<QueryPlaceholder.Parameter> parms = hibernateCriteria.getQuery().getParameters();
        final NakedObjectSpecification[] specs = new NakedObjectSpecification[parms.size()];
        final Naked[] objects = new Naked[parms.size()];
        for (int i = 0; i < parms.size(); i++) {
            final QueryPlaceholder.Parameter parm = parms.get(i);
            final NakedObject nakedObject = NakedObjectsContext.getObjectLoader().getAdapterFor(parm.getValue());
            specs[i] = nakedObject.getSpecification();
            objects[i] = nakedObject;
        }
        final Data[] objectData = (Data[]) encoder.createParameters(specs, objects, new KnownObjects());
        final byte[] serialisedQuery = serialise(hibernateCriteria.getQuery());
        return new HibernateQueryData((HibernateQueryCriteria) criteria, serialisedQuery, objectData);
    }

    // REVIEW serialise query inline here as part of strategy rather than in Marshaller - correct ? 
    private byte[] serialise(final Object toSerialise) {
        try {
            final ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            final ObjectOutputStream objectStream = new ObjectOutputStream(byteStream);
            objectStream.writeObject(toSerialise);
            objectStream.flush();
            return byteStream.toByteArray();
        } catch (Exception e) {
            throw new NakedObjectRuntimeException(e);
        }
    }

    private Object deSerialise(final byte[] bytes) {
        try {
            final ByteArrayInputStream byteStream = new ByteArrayInputStream(bytes);
            final ObjectInputStream objectStream = new ObjectInputStream(byteStream);
            return objectStream.readObject();
        } catch (Exception e) {
            throw new NakedObjectRuntimeException(e);
        }
    }

    protected InstancesCriteria doRestore(
            final NakedObjectSpecification specification,
            final boolean includeSubclasses,
            final CriteriaData criteriaData,
            final ObjectEncoder encoder) {

        final HibernateQueryData queryData = (HibernateQueryData) criteriaData;
        final QueryPlaceholder query = (QueryPlaceholder) deSerialise(queryData.getQueryAsBytes());

        for (int i = 0; i < queryData.getData().length; i++) {
            final QueryPlaceholder.Parameter parm = query.getParameters().get(i);
            final Data data = queryData.getData()[i];
            final Naked naked = (Naked) encoder.restore(data);
            parm.setValue(naked.getObject());
        }

        return new HibernateQueryCriteria(criteriaData.getCriteriaClass(), query, ((HibernateQueryData) criteriaData)
                .getResultType());
    }

    public Class getCriteriaClass() {
        return HibernateQueryCriteria.class;
    }

}
